'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global browser, translate, parseUri, BGcall */
(function ytchannelFunction() {
  let listenersAdded;

  // Store actual URL
  const url = document.location.href;

  // used to decode all encoded HTML  (convert '&' to &amp;)
  const parseElem = document.createElement('textarea');

  const parseChannelName = function (channelName) {
    function fixedEncodeURIComponent(str) {
      return encodeURIComponent(str).replace(/[!'()*]/g, c => `%${c.charCodeAt(0).toString(16)}`);
    }

    parseElem.innerHTML = channelName;
    const newChannelName = parseElem.innerText;

    // Remove whitespace, and encode
    return fixedEncodeURIComponent(newChannelName.replace(/\s/g, ''));
  };

  // Get id of the channel
  function getChannelId(aUrl) {
    return parseUri(aUrl).pathname.split('/')[2];
  }

  // Get id of the video
  function getVideoId(aUrl) {
    return parseUri.parseSearch(aUrl).v;
  }

  // Function which: - adds name of the channel on the end of the URL,
  //                   e.g. &ab_channel=nameofthechannel
  //                 - reload the page, so AdBlock can properly whitelist
  //                   the page (just if channel is whitelisted by user)
  function updateURL(channelName, shouldReload) {
    const parsedChannelName = parseChannelName(channelName);
    let updatedUrl;
    if (parseUri(url).search.indexOf('?') === -1) {
      updatedUrl = `${url}?&ab_channel=${parsedChannelName}`;
    } else {
      updatedUrl = `${url}&ab_channel=${parsedChannelName}`;
    }

    browser.runtime.sendMessage({ command: 'updateYouTubeChannelName', args: channelName });

    // Add the name of the channel to the end of URL
    window.history.replaceState(null, null, updatedUrl);

    // |shouldReload| is true only if we are not able to get
    // name of the channel by using YouTube Data v3 API
    if (shouldReload) {
      // Reload page from cache, if it should be whitelisted
      BGcall('pageIsWhitelisted', updatedUrl, (whitelisted) => {
        if (whitelisted) {
          document.location.reload(false);
        }
      });
    }
  }

  // Process messages from the background
  // - Query the YouTube page for channel names, and send them to the background page to white-list
  function onMessage(msg) {
    if (msg.type === 'whitelistAllYouTubeChannels') {
      // eslint-disable-next-line no-alert
      alert(translate('one_moment_all_youtube_channels'));

      // The list of subscriptions maybe accessed with different query selectors,
      // We'll try both.
      let myElements = document.querySelectorAll('a[href^="/channel/"');
      const titles = [];
      for (let inx = 0; inx < myElements.length; inx += 1) {
        const theChannelAnchorElement = myElements[inx];
        const theTitle = theChannelAnchorElement.getAttribute('title');
        if (theTitle && (typeof theTitle === 'string')) {
          titles.push(parseChannelName(theTitle.trim()));
        }
      }

      myElements = document.querySelectorAll('#guide-channels .display-name span');
      for (let inx = 0; inx < myElements.length; inx += 1) {
        const theElement = myElements[inx];
        const theTitle = theElement.innerText;
        if (theTitle) {
          titles.push(parseChannelName(theTitle.trim()));
        }
      }

      if (titles.length > 0) {
        // Remove any duplicates
        const seen = {};
        const newTitles = [];
        for (let inx = 0; inx < titles.length; inx += 1) {
          if (!(titles[inx] in seen)) {
            newTitles.push(titles[inx]);
            seen[titles[inx]] = true;
          }
        }

        browser.runtime.sendMessage({ command: 'updateYouTubeWhitelistFilters', args: newTitles }, () => {
          // eslint-disable-next-line no-alert
          alert(translate('all_youtube_channels_complete'));
        });
      }
    }
  }

  if (!/ab_channel/.test(url)) {
    // Get name of the channel by using YouTube Data v3 API
    if (/channel/.test(url)) {
      const xhr = new XMLHttpRequest();
      xhr.open(
        'GET',
        `https://www.googleapis.com/youtube/v3/channels?part=snippet&id=${getChannelId(url)
        }&key=${atob('QUl6YVN5QzJKMG5lbkhJZ083amZaUUYwaVdaN3BKd3dsMFczdUlz')}`,
      );
      xhr.onload = function onloadFunction() {
        if (xhr.readyState === 4 && xhr.status === 200) {
          const json = JSON.parse(xhr.response);

          // Got name of the channel
          if (json.items[0]) {
            updateURL(json.items[0].snippet.title, false);
          }
        }
      };

      xhr.send();
    } else if (/watch/.test(url)) {
      const xhr = new XMLHttpRequest();
      xhr.open(
        'GET',
        `https://www.googleapis.com/youtube/v3/videos?part=snippet&id=${getVideoId(url)
        }&key=${atob('QUl6YVN5QzJKMG5lbkhJZ083amZaUUYwaVdaN3BKd3dsMFczdUlz')}`,
      );
      xhr.onload = function onloadFunction() {
        if (xhr.readyState === 4 && xhr.status === 200) {
          const json = JSON.parse(xhr.response);

          // Got name of the channel
          if (json.items[0]) {
            updateURL(json.items[0].snippet.channelTitle, false);
          }
        }
      };

      xhr.send();
    } else if (/user/.test(url)) {
      document.addEventListener('spfdone', () => {
        const channelNameElement = document.querySelector('span .qualified-channel-title-text > a');
        if (channelNameElement && channelNameElement.textContent) {
          updateURL(channelNameElement.textContent, true);
        }
      }, true);

      // Spfdone event doesn't fire, when you access YT user directly
      window.addEventListener('DOMContentLoaded', () => {
        const channelNameElement = document.querySelector('span .qualified-channel-title-text > a');
        if (channelNameElement && channelNameElement.textContent) {
          updateURL(channelNameElement.textContent, true);
        }
      }, true);
    }
  }

  if ((typeof listenersAdded) === 'undefined') {
    listenersAdded = true;
    browser.runtime.onMessage.addListener(onMessage);
  }
}());
